using System;
using System.Reflection;

namespace DemoApp
{
	/// <summary>
	/// Summary description for ImportExportFactory.
	/// </summary>
	public class ImportExportFactory
	{
		public ImportExportFactory()
		{
		}

		private void Teste()
		{
			string dllName = @"c:\temp\DummyProvider.dll";

			// carregar assembly desejado
			Assembly asb = Assembly.LoadFrom(dllName);
			if (asb == null)
				throw new ApplicationException();

			// criar tipo desejado
			ImportExport.IImportExport obj = (ImportExport.IImportExport)asb.CreateInstance("DummyProvider.Class1");

			// obter meta tipo
			Type typ = asb.GetType("DummyProvider.Class1");
			// obter constructor sem parametros
			ConstructorInfo ctor = typ.GetConstructor(Type.EmptyTypes);
			// criar nova instancia por reflexo
			object obj2 = ctor.Invoke(null);
			// obter mtodo desejado
			MethodInfo mi = typ.GetMethod("MeuMetodo", new Type[]{typeof(int)});
			// invocar o mtodo com os argumentos desejados na instnica criada
			mi.Invoke(obj2, new object[]{123});
		}

		public ImportExport.IImportExport CreateProvider()
		{
			// ler ficheiro de configurao
			string providerName = System.Configuration.ConfigurationSettings.AppSettings["ImportExportProvider"];
			if (providerName == null || providerName.Length == 0)
				throw new ApplicationException("Falta configurar o provider de importao/exportao");

			try
			{
				// carregar assembly desejado
				Assembly asb = Assembly.LoadFrom(providerName);

				// pesquisar classes que implementam a interface desejada
				foreach (Type typ in asb.GetExportedTypes())
				{
					// se este tipo implementa a interface desejada
					if (typeof(ImportExport.IImportExport).IsAssignableFrom(typ))
					{
						// criar objecto fornecedor de servio

						// 1. obter construtor sem parametros
						ConstructorInfo ctor = typ.GetConstructor(Type.EmptyTypes);
						// 2. requisitar nova instncia deste tipo
						object obj = ctor.Invoke(null);

						return (ImportExport.IImportExport)obj;
					}
				}
			}
			catch (System.IO.FileNotFoundException ex)
			{
				throw new ApplicationException("provider no encontrado: " + providerName);
			}

			// no encontrou nehum tipo que implementasse a interface desejada
			return null;
		}
	}
}
